#ifndef AUBO_SDK_REGISTER_CONTROL_INTERFACE_H
#define AUBO_SDK_REGISTER_CONTROL_INTERFACE_H

#include <stdint.h>
#include <memory>
#include <vector>

#include <aubo/global_config.h>

namespace arcs {
namespace common_interface {

/**
 * 通用寄存器
 */
class ARCS_ABI_EXPORT RegisterControl
{
public:
    RegisterControl();
    virtual ~RegisterControl();

    /// Reads the boolean from one of the input registers, which can also be
    /// accessed by a Field bus. Note, uses it’s own memory space.
    ///
    /// @param address: Address of the register (0:127)
    /// @return The boolean value held by the register (true, false)
    ///
    /// @note he lower range of the boolean input registers [0:63] is reserved
    /// for FieldBus/PLC interface usage. The upper range [64:127] cannot be
    /// accessed by FieldBus/PLC interfaces, since it is reserved for external
    /// RTDE clients.
    bool getBoolInput(uint32_t address);

    /// @note 只有在实现 RTDE/Modbus Slave/PLC 服务端时使用
    int setBoolInput(uint32_t address, bool value);

    /// Reads the integer from one of the input registers, which can also be
    /// accessed by a Field bus. Note, uses it’s own memory space.
    ///
    /// @param address: Address of the register (0:47)
    /// @return The value held by the register [-2,147,483,648 : 2,147,483,647]
    /// @note The lower range of the integer input registers [0:23] is reserved
    /// for FieldBus/PLC interface usage. The upper range [24:47] cannot be
    /// accessed by FieldBus/PLC interfaces, since it is reserved for external
    /// RTDE clients.
    int getInt32Input(uint32_t address);

    /// @note 只有在实现 RTDE/Modbus Slave/PLC 服务端时使用
    int setInt32Input(uint32_t address, int value);

    /// Reads the float from one of the input registers, which can also be
    /// accessed by a Field bus. Note, uses it’s own memory space.
    ///
    /// @param address: Address of the register (0:47)
    /// @return The value held by the register (float)
    ///
    /// @note: The lower range of the float input registers [0:23] is reserved
    /// for FieldBus/PLC interface usage. The upper range [24:47] cannot be
    /// accessed by FieldBus/PLC interfaces, since it is reserved for external
    /// RTDE clients.
    float getFloatInput(uint32_t address);

    /// @note 只有在实现 RTDE/Modbus Slave/PLC 服务端时使用
    int setFloatInput(uint32_t address, float value);

    double getDoubleInput(uint32_t address);

    /// @note 只有在实现 RTDE/Modbus Slave/PLC 服务端时使用
    int setDoubleInput(uint32_t address, double value);

    /// Reads the boolean from one of the output registers, which can also be
    /// accessed by a Field bus.
    /// Note, uses it’s own memory space.
    ///
    /// @param address: Address of the register (0:127)
    /// @return The boolean value held by the register (true, false)
    ///
    /// @note The lower range of the boolean output registers [0:63] is reserved
    /// for FieldBus/PLC interface usage. The upper range [64:127] cannot be
    /// accessed by FieldBus/PLC interfaces, since it is reserved for external
    /// RTDE clients
    bool getBoolOutput(uint32_t address);
    int setBoolOutput(uint32_t address, bool value);

    /// Reads the integer from one of the output registers, which can also be
    /// accessed by a Field bus. Note, uses it’s own memory space.
    ///
    /// @param address: Address of the register (0:47)
    /// @return The int value held by the register [-2,147,483,648 :
    /// 2,147,483,647]
    ///
    /// @note The lower range of the integer output registers [0:23] is reserved
    /// for FieldBus/PLC interface usage. The upper range [24:47] cannot be
    /// accessed by FieldBus/PLC interfaces, since it is reserved for external
    /// RTDE clients.
    int getInt32Output(uint32_t address);
    int setInt32Output(uint32_t address, int value);

    /// Reads the float from one of the output registers, which can also be
    /// accessed by a Field bus. Note, uses it’s own memory space.
    ///
    /// @param address: Address of the register (0:47)
    /// @return The value held by the register (float)
    ///
    /// @note The lower range of the float output registers [0:23] is reserved
    /// for FieldBus/PLC interface usage. The upper range [24:47] cannot be
    /// accessed by FieldBus/PLC interfaces, since it is reserved for external
    /// RTDE clients.
    float getFloatOutput(uint32_t address);
    int setFloatOutput(uint32_t address, float value);

    double getDoubleOutput(uint32_t address);
    int setDoubleOutput(uint32_t address, double value);

    /// 用于 Modbus Slave
    int16_t getInt16Register(uint32_t address);
    int setInt16Register(uint32_t address, int16_t value);

    bool variableUpdated(const std::string &key, uint64_t since);

    /// 通用全局变量
    bool getBool(const std::string &key, bool default_value);
    int setBool(const std::string &key, bool value);

    std::vector<char> getVecChar(const std::string &key,
                                 const std::vector<char> &default_value);
    int setVecChar(const std::string &key, const std::vector<char> &value);

    int getInt32(const std::string &key, int default_value);
    int setInt32(const std::string &key, int value);

    std::vector<int32_t> getVecInt32(const std::string &key,
                                     const std::vector<int32_t> &default_value);
    int setVecInt32(const std::string &key, const std::vector<int32_t> &value);

    float getFloat(const std::string &key, float default_value);
    int setFloat(const std::string &key, float value);

    std::vector<float> getVecFloat(const std::string &key,
                                   const std::vector<float> &default_value);
    int setVecFloat(const std::string &key, const std::vector<float> &value);

    double getDouble(const std::string &key, double default_value);
    int setDouble(const std::string &key, double value);

    std::vector<double> getVecDouble(const std::string &key,
                                     const std::vector<double> &default_value);
    int setVecDouble(const std::string &key, const std::vector<double> &value);

    std::string getString(const std::string &key,
                          const std::string &default_value);
    int setString(const std::string &key, const std::string &value);

    int clearNamedVariable(const std::string &key);

    /// Adds a new modbus signal for the controller to supervise. Expects no
    /// response.
    ///
    /// @param IP: A string specifying the IP address of the modbus unit to
    /// which the modbus signal is connected
    /// @param slave_number: An integer normally not used and set to 255, but is
    /// a free choice between 0 and 255.
    /// @param signal_address: An integer specifying the address of the either
    /// the coil or the register that this new signal should reflect. Consult
    /// the configuration of the modbus unit for this information.
    /// @param signal_type: An integer specifying the type of signal to add. 0 =
    /// digital input, 1 = digital output, 2 = register input and 3 = register
    /// output.
    /// @param signal_name: A string uniquely identifying the signal. If a
    /// string is supplied which is equal to an already added signal, the new
    /// signal will replace the old one. The length of the string cannot exceed
    /// 20 characters.
    /// @param sequential_mode: Setting to True forces the modbus client to wait
    /// for a response before sending the next request. This mode is required by
    /// some fieldbus units (Optional).
    int modbusAddSignal(const std::string &IP, int slave_number,
                        int signal_address, int signal_type,
                        const std::string &signal_name, bool sequential_mode);

    /// Deletes the signal identified by the supplied signal name.
    /// @param signal_name: A string equal to the name of the signal that should
    /// be deleted.
    int modbusDeleteSignal(const std::string &signal_name);

    /// Reads the current value of a specific signal.
    ///
    /// @param signal_name: A string equal to the name of the signal for which
    /// the value should be gotten.
    ///
    /// @return An integer or a boolean. For digital signals: 1 or 0. For
    /// register signals: The register value expressed as an unsigned integer.
    uint16_t modbusGetSignalStatus(const std::string &signal_name);

    /// Sends a command specified by the user to the modbus unit located on the
    /// specified IP address. Cannot be used to request data, since the response
    /// will not be received. The user is responsible for supplying data which
    /// is meaningful to the supplied function code. The builtin function takes
    /// care of constructing the modbus frame, so the user should not be
    /// concerned with the length of the command.
    ///
    /// @param IP: A string specifying the IP address locating the modbus unit
    /// to which the custom command should be send.
    /// @param slave_number: An integer specifying the slave number to use for
    /// the custom command.
    /// @param function_code: An integer specifying the function code for the
    /// custom command.
    /// @param data: An array of integers in which each entry must be a valid
    /// byte (0-255) value.
    int modbusSendCustomCommand(const std::string &IP, int slave_number,
                                int function_code,
                                const std::vector<uint8_t> &data);

    /// Sets the selected digital input signal to either a "default" or
    /// "freedrive" action.
    /// @param signal_name: A string identifying a digital input signal that was
    /// previously added.
    /// @param action: The type of action. The action can either be "default" or
    /// "freedrive". (string)
    int modbusSetDigitalInputAction(const std::string &signal_name, int action);

    /// Sets the output register signal identified by the given name to the
    /// given value.
    /// @param signal_name: A string identifying an output register signal that
    /// in advance has been added.
    /// @param value: An integer which must be a valid word (0-65535)
    /// value.
    int modbusSetOutputSignal(const std::string &signal_name, uint16_t value);

    /// Sets the frequency with which the robot will send requests to the Modbus
    /// controller to either read or write the signal value.
    /// @param signal_name: A string identifying an output digital signal that
    /// in advance has been added.
    /// @param update_frequency: An integer in the range 0-125 specifying the
    /// update frequency in Hz.
    int modbusSetSignalUpdateFrequency(const std::string &signal_name,
                                       int update_frequency);

protected:
    void *d_;
};
using RegisterControlPtr = std::shared_ptr<RegisterControl>;

// clang-format off
#define REGISTER_CONTROL_DECLARES                           \
    X(RegisterControl, 1, getBoolInput, address)            \
    X(RegisterControl, 2, setBoolInput, address, value)     \
    X(RegisterControl, 1, getInt32Input, address)           \
    X(RegisterControl, 2, setInt32Input, address, value)    \
    X(RegisterControl, 1, getFloatInput, address)           \
    X(RegisterControl, 2, setFloatInput, address, value)    \
    X(RegisterControl, 1, getDoubleInput, address)          \
    X(RegisterControl, 2, setDoubleInput, address, value)   \
    X(RegisterControl, 1, getBoolOutput, address)           \
    X(RegisterControl, 2, setBoolOutput, address, value)    \
    X(RegisterControl, 1, getInt32Output, address)          \
    X(RegisterControl, 2, setInt32Output, address, value)   \
    X(RegisterControl, 1, getFloatOutput, address)          \
    X(RegisterControl, 2, setFloatOutput, address, value)   \
    X(RegisterControl, 1, getDoubleOutput, address)         \
    X(RegisterControl, 2, setDoubleOutput, address, value)  \
    X(RegisterControl, 1, getInt16Register, address)        \
    X(RegisterControl, 2, setInt16Register, address, value) \
    X(RegisterControl, 2, variableUpdated, key, since)      \
    X(RegisterControl, 2, getBool, key, default_value)      \
    X(RegisterControl, 2, setBool, key, value)              \
    X(RegisterControl, 2, getVecChar, key, default_value)   \
    X(RegisterControl, 2, setVecChar, key, value)           \
    X(RegisterControl, 2, getInt32, key, default_value)     \
    X(RegisterControl, 2, setInt32, key, value)             \
    X(RegisterControl, 2, getVecInt32, key, default_value)  \
    X(RegisterControl, 2, setVecInt32, key, value)          \
    X(RegisterControl, 2, getFloat, key, default_value)     \
    X(RegisterControl, 2, setFloat, key, value)             \
    X(RegisterControl, 2, getVecFloat, key, default_value)  \
    X(RegisterControl, 2, setVecFloat, key, value)          \
    X(RegisterControl, 2, getDouble, key, default_value)    \
    X(RegisterControl, 2, setDouble, key, value)            \
    X(RegisterControl, 2, getVecDouble, key, default_value) \
    X(RegisterControl, 2, setVecDouble, key, value)         \
    X(RegisterControl, 2, getString, key, default_value)    \
    X(RegisterControl, 2, setString, key, value)            \
    X(RegisterControl, 1, clearNamedVariable, key)          \
    X(RegisterControl, 6, modbusAddSignal, IP, slave_number, signal_address, signal_type, signal_name, sequential_mode) \
    X(RegisterControl, 1, modbusDeleteSignal, signal_name) \
    X(RegisterControl, 1, modbusGetSignalStatus, signal_name) \
    X(RegisterControl, 4, modbusSendCustomCommand, IP, slave_number, function_code, data) \
    X(RegisterControl, 2, modbusSetDigitalInputAction, signal_name, action) \
    X(RegisterControl, 2, modbusSetOutputSignal, signal_name, value) \
    X(RegisterControl, 2, modbusSetSignalUpdateFrequency, signal_name, update_frequency)
// clang-format on
} // namespace common_interface
} // namespace arcs

#endif // AUBO_SDK_REGISTER_CONTROL_INTERFACE_H
