#ifndef AUBO_SDK_IO_CONTROL_INTERFACE_H
#define AUBO_SDK_IO_CONTROL_INTERFACE_H

#include <vector>
#include <thread>

#include <aubo/global_config.h>
#include <aubo/type_def.h>

namespace arcs {
namespace common_interface {

/**
 * IoControl
 * 提供了一系列的接口对机器人标配的一些数字、模拟IO进行配置，输出状态设置、读取
 *
 * 1. 获取各种IO的数量
 * 2. 配置IO的输入输出功能
 * 3. 可配置IO的配置
 * 4. 模拟IO的输入输出范围设置、读取
 *
 * 标准数字输入输出：控制柜IO面板上的标准IO
 * 工具端数字输入输出：通过工具末端航插暴露的数字IO
 * 可配置输入输出：可以配置为安全IO或者普通数字IO
 */
class ARCS_ABI_EXPORT IoControl
{
public:
    IoControl();
    virtual ~IoControl();

    /**
     * @return 返回标准数字输入数量
     */
    int getStandardDigitalInputNum();

    /**
     * @return 返回工具端数字输入数量
     */
    int getToolDigitalInputNum();

    /**
     * @return 返回可配置数字输入数量
     */
    int getConfigurableDigitalInputNum();

    /**
     * 获取标准数字输出数量
     */
    int getStandardDigitalOutputNum();

    /**
     * 获取工具端数字输出数量
     */
    int getToolDigitalOutputNum();

    /**
     * 设置指定的工具端IO为输入/输出
     *
     * 工具端IO比较特殊，IO可以配置为输入或者输出
     *
     * @param input 为true时，设置指定IO为输入，否则为输出
     */
    int setToolIoInput(int index, bool input);

    /**
     * @return 当指定的IO为输入时返回 true, 否则为 false
     */
    bool isToolIoInput(int index);

    /**
     * 获取可配置数字输出数量
     */
    int getConfigurableDigitalOutputNum();

    /**
     * 获取标准模拟输入数量
     */
    int getStandardAnalogInputNum();

    /**
     * 获取工具端模拟输入数量
     */
    int getToolAnalogInputNum();

    /**
     * 获取标准模拟输出数量
     */
    int getStandardAnalogOutputNum();

    /**
     * 获取工具端模拟输出数量
     */
    int getToolAnalogOutputNum();

    /**
     * 设置所有数字输入动作为默认值
     *
     * @return
     */
    int setDigitalInputActionDefault();

    /**
     * 设置标准数字输入触发动作
     */
    int setStandardDigitalInputAction(int index, StandardInputAction action);

    /**
     * 设置工具数字输入触发动作
     */
    int setToolDigitalInputAction(int index, StandardInputAction action);

    /**
     * 设置可配置数字输入触发动作
     *
     * @note 需要将可配置输入的安全输入动作设置为
     * SafetyInputAction::Unassigned时这个函数的配置才会生效
     */
    int setConfigurableDigitalInputAction(int index,
                                          StandardInputAction action);

    /**
     * 获取标准数字输入的输入触发动作
     */
    StandardInputAction getStandardDigitalInputAction(int index);

    /**
     * 获取工具端数字输入的输入触发动作
     */
    StandardInputAction getToolDigitalInputAction(int index);

    /**
     * 获取可配置数字输入的输入触发动作
     */
    StandardInputAction getConfigurableDigitalInputAction(int index);

    /**
     * 设置所有数字输出动作为默认值
     *
     * @return
     */
    int setDigitalOutputRunstateDefault();

    /**
     * 设置标准数字输出状态选择
     */
    int setStandardDigitalOutputRunstate(int index,
                                         StandardOutputRunState runstate);

    /**
     * 设置工具端数字输出状态选择
     */
    int setToolDigitalOutputRunstate(int index,
                                     StandardOutputRunState runstate);

    /**
     * 设置可配置数字输出状态选择
     */
    int setConfigurableDigitalOutputRunstate(int index,
                                             StandardOutputRunState runstate);

    /**
     * 获取标准数字输出状态选择
     */
    StandardOutputRunState getStandardDigitalOutputRunstate(int index);

    /**
     * 获取工具端数字输出状态选择
     */
    StandardOutputRunState getToolDigitalOutputRunstate(int index);

    /**
     * 获取可配置数字输出状态选择
     */
    StandardOutputRunState getConfigurabDigitalOutputRunstate(int index);

    /**
     * 设置标准模拟输出状态选择
     */
    int setStandardAnalogOutputRunstate(int index,
                                        StandardOutputRunState runstate);

    /**
     * 设置工具端模拟输出状态选择
     */
    int setToolAnalogOutputRunstate(int index, StandardOutputRunState runstate);

    /**
     * 获取标准模拟输出状态选择
     */
    StandardOutputRunState getStandardAnalogOutputRunstate(int index);

    /**
     * 获取工具端模拟输出状态选择
     */
    StandardOutputRunState getToolAnalogOutputRunstate(int index);

    /**
     * 设置模拟输入的范围
     */
    int setStandardAnalogInputDomain(int index, int domain);
    int setToolAnalogInputDomain(int index, int domain);

    /**
     * 获取标准模式输入范围
     */
    int getStandardAnalogInputDomain(int index);

    /**
     * 获取工具端模式输入范围
     */
    int getToolAnalogInputDomain(int index);

    /**
     * 设置标准模拟输出的范围
     */
    int setStandardAnalogOutputDomain(int index, int domain);

    /**
     * 设置工具端模拟输出范围
     */
    int setToolAnalogOutputDomain(int index, int domain);

    /**
     * 获取标准模拟输出范围
     */
    int getStandardAnalogOutputDomain(int index);

    /**
     * 获取工具端模拟输出范围
     */
    int getToolAnalogOutputDomain(int index);

    /**
     * 设置标准数字输出
     */
    int setStandardDigitalOutput(int index, bool value);

    /**
     * 设置工具端数字输出
     */
    int setToolDigitalOutput(int index, bool value);

    /**
     * 设置可配置数字输出
     */
    int setConfigurableDigitalOutput(int index, bool value);

    /**
     * 设置标准模拟输出
     */
    int setStandardAnalogOutput(int index, double value);

    /**
     * 设置工具端模拟输出
     */
    int setToolAnalogOutput(int index, double value);

    /**
     * 获取标准数字输入
     */
    bool getStandardDigitalInput(int index);
    uint32_t getStandardDigitalInputs();

    /**
     * 获取工具端数字输入
     */
    bool getToolDigitalInput(int index);
    uint32_t getToolDigitalInputs();

    /**
     * 获取可配置数字输入
     */
    bool getConfigurableDigitalInput(int index);
    uint32_t getConfigurableDigitalInputs();

    /**
     * 获取标准数字输出
     */
    bool getStandardDigitalOutput(int index);
    uint32_t getStandardDigitalOutputs();

    /**
     * 获取工具端数字输出
     */
    bool getToolDigitalOutput(int index);
    uint32_t getToolDigitalOutputs();

    /**
     * 获取可配置数字输出
     */
    bool getConfigurableDigitalOutput(int index);
    uint32_t getConfigurableDigitalOutputs();

    /**
     * 获取标准模拟输入
     */
    double getStandardAnalogInput(int index);

    /**
     * 获取工具端模拟输入
     */
    double getToolAnalogInput(int index);

    /**
     * 获取标准模拟输出
     */
    double getStandardAnalogOutput(int index);

    /**
     * 获取工具端模拟输出
     */
    double getToolAnalogOutput(int index);

    /**
     * 兼容接口板，仅供读数显示
     */
    int getStaticSafetyInputNum();
    int getStaticSafetyOutputNum();
    uint32_t getStaticSafetyInputs();
    uint32_t getStaticSafetyOutputs();

    int getStaticLinkInputNum();
    int getStaticLinkOutputNum();
    uint32_t getStaticLinkInputs();
    uint32_t getStaticLinkOutputs();

protected:
    void *d_;
};
using IoControlPtr = std::shared_ptr<IoControl>;

// clang-format off
#define IO_CONTROL_DECLARES                                              \
    X(IoControl, 0, getStandardDigitalInputNum)                          \
    X(IoControl, 0, getToolDigitalInputNum)                              \
    X(IoControl, 0, getConfigurableDigitalInputNum)                      \
    X(IoControl, 0, getStandardDigitalOutputNum)                         \
    X(IoControl, 0, getToolDigitalOutputNum)                             \
    X(IoControl, 2, setToolIoInput, index, input)                        \
    X(IoControl, 1, isToolIoInput, index)                                \
    X(IoControl, 0, getConfigurableDigitalOutputNum)                     \
    X(IoControl, 0, getStandardAnalogInputNum)                           \
    X(IoControl, 0, getToolAnalogInputNum)                               \
    X(IoControl, 0, getStandardAnalogOutputNum)                          \
    X(IoControl, 0, getToolAnalogOutputNum)                              \
    X(IoControl, 0, setDigitalInputActionDefault)                        \
    X(IoControl, 2, setStandardDigitalInputAction, index, action)        \
    X(IoControl, 2, setToolDigitalInputAction, index, action)            \
    X(IoControl, 2, setConfigurableDigitalInputAction, index, action)    \
    X(IoControl, 1, getStandardDigitalInputAction, index)                \
    X(IoControl, 1, getToolDigitalInputAction, index)                    \
    X(IoControl, 1, getConfigurableDigitalInputAction, index)            \
    X(IoControl, 0, setDigitalOutputRunstateDefault)                     \
    X(IoControl, 2, setStandardDigitalOutputRunstate, index, runstate)   \
    X(IoControl, 2, setToolDigitalOutputRunstate, index, runstate)       \
    X(IoControl, 2, setConfigurableDigitalOutputRunstate, index, runstate) \
    X(IoControl, 1, getStandardDigitalOutputRunstate, index)             \
    X(IoControl, 1, getToolDigitalOutputRunstate, index)                 \
    X(IoControl, 1, getConfigurabDigitalOutputRunstate, index)           \
    X(IoControl, 2, setStandardAnalogOutputRunstate, index, runstate)    \
    X(IoControl, 2, setToolAnalogOutputRunstate, index, runstate)        \
    X(IoControl, 1, getStandardAnalogOutputRunstate, index)              \
    X(IoControl, 1, getToolAnalogOutputRunstate, index)                  \
    X(IoControl, 2, setStandardAnalogInputDomain, index, domain)         \
    X(IoControl, 2, setToolAnalogInputDomain, index, domain)             \
    X(IoControl, 1, getStandardAnalogInputDomain, index)                 \
    X(IoControl, 1, getToolAnalogInputDomain, index)                     \
    X(IoControl, 2, setStandardAnalogOutputDomain, index, domain)        \
    X(IoControl, 2, setToolAnalogOutputDomain, index, domain)            \
    X(IoControl, 1, getStandardAnalogOutputDomain, index)                \
    X(IoControl, 1, getToolAnalogOutputDomain, index)                    \
    X(IoControl, 2, setStandardDigitalOutput, index, value)              \
    X(IoControl, 2, setToolDigitalOutput, index, value)                  \
    X(IoControl, 2, setConfigurableDigitalOutput, index, value)          \
    X(IoControl, 2, setStandardAnalogOutput, index, value)               \
    X(IoControl, 2, setToolAnalogOutput, index, value)                   \
    X(IoControl, 1, getStandardDigitalInput, index)                      \
    X(IoControl, 0, getStandardDigitalInputs)                            \
    X(IoControl, 1, getToolDigitalInput, index)                          \
    X(IoControl, 0, getToolDigitalInputs)                                \
    X(IoControl, 1, getConfigurableDigitalInput, index)                  \
    X(IoControl, 0, getConfigurableDigitalInputs)                        \
    X(IoControl, 1, getStandardAnalogInput, index)                       \
    X(IoControl, 1, getToolAnalogInput, index)                           \
    X(IoControl, 1, getStandardDigitalOutput, index)                     \
    X(IoControl, 0, getStandardDigitalOutputs)                           \
    X(IoControl, 1, getToolDigitalOutput, index)                         \
    X(IoControl, 0, getToolDigitalOutputs)                               \
    X(IoControl, 1, getConfigurableDigitalOutput, index)                 \
    X(IoControl, 0, getConfigurableDigitalOutputs)                       \
    X(IoControl, 1, getStandardAnalogOutput, index)                      \
    X(IoControl, 1, getToolAnalogOutput, index)                          \
    X(IoControl, 0, getStaticSafetyInputNum)                             \
    X(IoControl, 0, getStaticSafetyOutputNum)                            \
    X(IoControl, 0, getStaticSafetyInputs)                               \
    X(IoControl, 0, getStaticSafetyOutputs)                              \
    X(IoControl, 0, getStaticLinkInputNum)                               \
    X(IoControl, 0, getStaticLinkOutputNum)                              \
    X(IoControl, 0, getStaticLinkInputs)                                 \
    X(IoControl, 0, getStaticLinkOutputs)
// clang-format on
} // namespace common_interface
} // namespace arcs

#endif // AUBO_SDK_IO_CONTROL_INTERFACE_H
