#ifndef AUBO_SDK_MOTION_CONTROL_INTERFACE_H
#define AUBO_SDK_MOTION_CONTROL_INTERFACE_H

#include <vector>
#include <functional>
#include <thread>

#include <aubo/global_config.h>
#include <aubo/type_def.h>

/**
 * @file motion_control.h
 *
 * The robot movements are programmed as pose-to-pose movements, that is move
 * from the current position to a new position. The path between these two
 * positions is then automatically calculated by the robot.
 */
namespace arcs {
namespace common_interface {

/**
 * @brief The MotionControl class
 */
class ARCS_ABI_EXPORT MotionControl
{
public:
    MotionControl();
    virtual ~MotionControl();

    /**
     * 获取等效半径，单位m
     * moveLine/moveCircle时，末端姿态旋转的角度等效到末端位置移动
     */
    double getEqradius();

    /**
     * 动态调整机器人运行速度
     */
    int setSpeedFraction(double fraction);

    /**
     * 获取速度/加速度比例
     */
    double getSpeedFraction();

    /**
     * 路径偏移使能
     */
    int pathOffsetEnable();

    /**
     * 设置路径偏移
     * offset: 在各方向的位姿偏移
     */
    int pathOffsetSet(const std::vector<double> &offset, int type = 1);

    /**
     * 路径偏移失能
     */
    int pathOffsetDisable();

    /**
     * 关节偏移使能
     */
    int jointOffsetEnable();

    /**
     * 设置关节偏移
     * offset: 在各关节的位姿偏移
     */
    int jointOffsetSet(const std::vector<double> &offset, int type = 1);

    /**
     * 关节偏移失能
     */
    int jointOffsetDisable();

    /**
     * 获取已经入队的运动指令段数量
     */
    int getQueueSize();

    /**
     * 获取已经入队的运动规划插补点数量
     */
    int getTrajectoryQueueSize();

    /**
     * 获取当前正在插补的运动指令段的ID
     */
    int getExecId();

    /**
     * 获取下一个插补的运动指令段的ID
     */
    int getNextExecId();

    /**
     * 获取指定ID的运动指令段的预期执行时间
     */
    double getDuration(int id);

    /**
     * 获取指定ID的运动指令段的剩余执行时间
     */
    double getMotionLeftTime(int id);

    /**
     * 获取指定ID的运动指令段的执行进度
     */
    double getProgress(int id);

    /**
     * 当工件安装在另外一台机器人的末端或者外部轴上时，指定其名字和安装位置
     *
     * @param module_name 控制模块名字
     * @param mounting_pose
     * 抓取的相对位置，如果是机器人，则相对于机器人末端中心点（非TCP点）
     * @return
     */
    int setWorkObjectHold(const std::string &module_name,
                          const std::vector<double> &mounting_pose);

    /**
     * getWorkObjectHold
     *
     * @return
     */
    std::tuple<std::string, std::vector<double>> getWorkObjectHold();

    /**
     * 关节空间伺服
     */
    int servoJoint(const std::vector<double> &q, double a, double v, double t,
                   double lookahead_time, double gain);

    /**
     * 关节空间跟随
     */
    int followJoint(const std::vector<double> &q);

    /**
     * 笛卡尔空间跟随
     */
    int followLine(const std::vector<double> &pose);

    /**
     * 关节空间速度跟随
     */
    int speedJoint(const std::vector<double> &q, double a, double t);

    /**
     * 笛卡尔空间速度跟随
     */
    int speedLine(const std::vector<double> &pose, double a, double t);

    /**
     * 在圆弧上进行匀速运动
     *
     * @return
     */
    /* int speedCircle(const std::vector<double> &via_pose,
                            const std::vector<double> &end_pose,
                            const std::vector<double> &twist, double a,
                            bool direction, double t) ;*/

    /**
     * 添加moveSpline轨迹点
     */
    int moveSpline(const std::vector<double> &q, double duration);

    /**
     * 添加关节运动
     */
    int moveJoint(const std::vector<double> &q, double a, double v,
                  double blend_radius, double duration);

    /**
     * 添加直线运动
     */
    int moveLine(const std::vector<double> &pose, double a, double v,
                 double blend_radius, double duration);

    /**
     * 添加圆弧运动
     *
     * @param via_pose
     * @param end_pose
     * @param a
     * @param v
     * @param blend_radius
     * @param duration
     * angle =0: 圆弧运动
     * angle >0: 圆弧运动或者圆周运动，end_pose仅决定圆弧的形状，不决定其终止点
     * direction: 是否转换方向 curr->via->end(×)  or curr->end(√)
     */
    int moveCircle(const std::vector<double> &via_pose,
                   const std::vector<double> &end_pose, double a, double v,
                   double blend_radius, double duration);

    /**
     * @brief pathBufferAlloc
     * @param name
     * @param type
     * @param size
     * @return
     */
    int pathBufferAlloc(const std::string &name, int type, int size);

    /**
     * 向路径缓存添加路点
     */
    int pathBufferAppend(const std::string &name,
                         const std::vector<std::vector<double>> &waypoints);

    /**
     * 计算、优化，传入的参数相同时不会重新计算
     */
    bool pathBufferEval(const std::string &name, const std::vector<double> &a,
                        const std::vector<double> &v, double t);

    /**
     * 指定名字的buffer是否有效
     */
    bool pathBufferValid(const std::string &name);

    /**
     * 释放路径缓存
     */
    int pathBufferFree(const std::string &name);

    /**
     * 列出所有缓存路径的名字
     */
    std::vector<std::string> pathBufferList();

    /**
     * 执行缓存的路径
     */
    int movePathBuffer(const std::string &name);

    /**
     * 关节空间停止运动
     */
    int stopJoint(double acc);

    /**
     * 笛卡尔控制控制运动
     */
    int stopLine(double acc, double acc_rot);

protected:
    void *d_;
};
using MotionControlPtr = std::shared_ptr<MotionControl>;

// clang-format off
#define MOTION_CONTROL_DECLARES                                             \
    X(MotionControl, 0, getEqradius)                                        \
    X(MotionControl, 0, getSpeedFraction)                                   \
    X(MotionControl, 1, setSpeedFraction, fraction)                         \
    X(MotionControl, 0, pathOffsetEnable)                                   \
    X(MotionControl, 2, pathOffsetSet, offset, type)                        \
    X(MotionControl, 0, pathOffsetDisable)                                  \
    X(MotionControl, 0, jointOffsetEnable)                                  \
    X(MotionControl, 2, jointOffsetSet, offset, type)                       \
    X(MotionControl, 0, jointOffsetDisable)                                 \
    X(MotionControl, 0, getTrajectoryQueueSize)                             \
    X(MotionControl, 0, getQueueSize)                                       \
    X(MotionControl, 0, getExecId)                                          \
    X(MotionControl, 0, getNextExecId)                                      \
    X(MotionControl, 1, getDuration, id)                                    \
    X(MotionControl, 1, getMotionLeftTime, id)                              \
    X(MotionControl, 1, getProgress, id)                                    \
    X(MotionControl, 2, setWorkObjectHold, module_name, mounting_pose)      \
    X(MotionControl, 0, getWorkObjectHold)                                  \
    X(MotionControl, 6, servoJoint, q, a, v, t, lookahead_time, gain)       \
    X(MotionControl, 1, followJoint, q)                                     \
    X(MotionControl, 1, followLine, pose)                                   \
    X(MotionControl, 3, speedJoint, q, a, t)                                \
    X(MotionControl, 3, speedLine, pose, a, t)                              \
    X(MotionControl, 2, moveSpline, q, duration)                            \
    X(MotionControl, 5, moveJoint, q, a, v, blend_radius, duration)         \
    X(MotionControl, 5, moveLine, pose, a, v, blend_radius, duration)       \
    X(MotionControl, 6, moveCircle, via_pose, end_pose, a, v,               \
                                blend_radius, duration)                     \
    X(MotionControl, 3, pathBufferAlloc, name, type, size)                  \
    X(MotionControl, 2, pathBufferAppend, name, waypoints)                  \
    X(MotionControl, 4, pathBufferEval, name, a, v, t)                      \
    X(MotionControl, 1, pathBufferValid, name)                              \
    X(MotionControl, 1, pathBufferFree, name)                               \
    X(MotionControl, 0, pathBufferList)                                     \
    X(MotionControl, 1, movePathBuffer, name)                               \
    X(MotionControl, 1, stopJoint, acc)                                     \
    X(MotionControl, 2, stopLine, acc, acc_rot)
// clang-format on
} // namespace common_interface
} // namespace arcs

#endif // AUBO_SDK_MOTION_CONTROL_INTERFACE_H
