#ifndef AUBO_SDK_ROBOT_CONFIG_H
#define AUBO_SDK_ROBOT_CONFIG_H

#include <vector>
#include <unordered_map>

#include <aubo/global_config.h>
#include <aubo/type_def.h>

namespace arcs {
namespace common_interface {

class ARCS_ABI_EXPORT RobotConfig
{
public:
    RobotConfig();
    virtual ~RobotConfig();

    /**
     * @return 机器人的名字
     */
    std::string getName();

    /**
     * 获取机器人的自由度(从硬件抽象层读取)
     */
    int getDof();

    /**
     * 获取机器人的伺服控制周期(从硬件抽象层读取)
     */
    double getCycletime();

    /**
     * 获取默认的工具端加速度，单位m/s^2
     */
    double getDefaultToolAcc();

    /**
     * 获取默认的工具端速度，单位m/s
     */
    double getDefaultToolSpeed();

    /**
     * 获取默认的关节加速度，单位rad/s^2
     */
    double getDefaultJointAcc();

    /**
     * 获取默认的关节速度，单位rad/s
     */
    double getDefaultJointSpeed();

    /**
     * 获取机器人类型代码
     */
    std::string getRobotType();

    /**
     * 获取机器人子类型代码
     */
    std::string getRobotSubType();

    /**
     * 获取控制柜类型代码
     */
    int getControlBoxType();

    /**
     * 设置安装位姿(机器人的基坐标系相对于世界坐标系)  world->base
     * 一般在多机器人系统中使用，默认为 [0,0,0,0,0,0]
     *
     * @param pose
     * @return
     */
    int setMountingPose(const std::vector<double> &pose);

    /**
     * 获取安装位姿(机器人的基坐标系相对于世界坐标系)
     *
     * @return
     */
    std::vector<double> getMountingPose();

    /**
     * 设置碰撞灵敏度等级，数值越大越灵敏
     *
     * @param level 0: 关闭碰撞检测功能  1~9: 碰撞灵敏等级
     * @return
     */
    int setCollisionLevel(int level);

    /**
     * 获取碰撞灵敏度等级
     *
     * @return 碰撞灵敏度等级
     */
    int getCollisionLevel();

    /**
     * 设置碰撞停止类型
     *
     * @param type 0: 碰撞停机 1: 碰撞之后进入拖动模式
     * @return
     */
    int setCollisionStopType(int type);

    /**
     * 获取碰撞停止类型
     *
     * @return 碰撞停止类型
     */
    int getCollisionStopType();

    int setToolVoltageOutputDomain(int domain);
    int getToolVoltageOutputDomain();

    /**
     * 获取机器人DH参数
     * alpha a d theta beta
     *
     * @param real 读取真实参数(理论值+补偿值)或者理论参数
     */
    std::unordered_map<std::string, std::vector<double>> getKinematicsParam(
        bool real);

    /**
     * 获取指定温度下的DH参数补偿值
     * alpha a d theta beta
     *
     * @param ref_temperature 参考温度 ℃，默认20℃
     */
    std::unordered_map<std::string, std::vector<double>>
    getKinematicsCompensate(double ref_temperature);

    /**
     * 设置运动学模型补偿参数
     * da dalpha dd dtheta dbeta
     *
     * @param comp 补偿数据
     * @param ref_temperature 参考温度 ℃，默认20℃
     */
    int setPersistentParameters(const std::string &param);

    /**
     * 获取可用的末端力矩传感器的名字
     */
    std::vector<std::string> getTcpForceSensorNames();

    /**
     * 设置末端力矩传感器
     * 如果存在内置的末端力矩传感器，默认将使用内置的力矩传感器
     */
    int selectTcpForceSensor(const std::string &name);

    /**
     * 是否安装了末端力矩传感器
     */
    bool hasTcpForceSensor();

    /**
     * 设置末端力矩偏移
     */
    int setTcpForceOffset(const std::vector<double> &force_offset);

    /**
     * 获取末端力矩偏移
     */
    std::vector<double> getTcpForceOffset();

    /**
     * 获取可用的底座力矩传感器的名字
     */
    std::vector<std::string> getBaseForceSensorNames();

    /**
     * 设置底座力矩传感器
     * 如果存在内置的底座力矩传感器，默认将使用内置的力矩传感器
     */
    int selectBaseForceSensor(const std::string &name);

    /**
     * 是否安装了底座力矩传感器
     */
    bool hasBaseForceSensor();

    /**
     * 设置底座力矩偏移
     */
    int setBaseForceOffset(const std::vector<double> &force_offset);

    /**
     * 获取底座力矩偏移
     */
    std::vector<double> getBaseForceOffset();

    /**
     * 获取安全参数校验码 CRC32
     */
    uint32_t getSafetyParametersCheckSum();

    /**
     * 发起确认安全配置参数请求: 将安全配置参数写入到安全接口板flash或文件
     */
    int confirmSafetyParameters(const RobotSafetyParameterRange &parameters);

    /**
     * 获取关节最大位置（物理极限）
     */
    std::vector<double> getJointMaxPositions();

    /**
     * 获取关节最小位置（物理极限）
     */
    std::vector<double> getJointMinPositions();

    /**
     * 获取关节最大速度（物理极限）
     */
    std::vector<double> getJointMaxSpeeds();

    /**
     * 获取关节最大加速度（物理极限）
     */
    std::vector<double> getJointMaxAccelerations();

    /**
     * 获取TCP最大速度（物理极限）
     */
    std::vector<double> getTcpMaxSpeeds();

    /**
     * 获取TCP最大加速度（物理极限）
     */
    std::vector<double> getTcpMaxAccelerations();

    /**
     * 设置机器人安装姿态
     */
    int setGravity(const std::vector<double> &gravity);

    /**
     * 获取机器人的安装姿态，如果机器人底座安装了姿态传感器，则从传感器读取数据，否则按照用户设置
     */
    std::vector<double> getGravity();

    /**
     * 设置TCP偏移
     */
    int setTcpOffset(const std::vector<double> &offset);

    /**
     * 获取TCP偏移
     */
    std::vector<double> getTcpOffset();

    /**
     * 设置工具端质量、质心及惯量
     */
    int setToolInertial(double m, const std::vector<double> &com,
                        const std::vector<double> &inertial);

    /**
     * 设置末端负载
     */
    int setPayload(double m, const std::vector<double> &cog,
                   const std::vector<double> &aom,
                   const std::vector<double> &inertia);

    /**
     * 获取末端负载
     */
    Payload getPayload();

    /**
     * 末端位姿是否在安全范围之内
     */
    bool toolSpaceInRange(const std::vector<double> &pose);

    /**
     * 发起固件升级请求，控制器软件将进入固件升级模式
     *
     * @param fw 固件路径
     *     pm://param/model/xx.bin
     *     /absolute/path/to/xx.bin
     *     relative/path/to/xx.bin
     */
    int firmwareUpdate(const std::string &fw);

    /**
     * 获取固件升级的进程
     *
     * @return std::string 代表步骤名称, double 代表进度(0~1)，完成之后，返回
     * ("", 1)
     */
    std::tuple<std::string, double> getFirmwareUpdateProcess();

protected:
    void *d_;
};
using RobotConfigPtr = std::shared_ptr<RobotConfig>;

// clang-format off
#define ROBOT_CONFIG_DECLARES                                                \
    X(RobotConfig, 0, getDof)                                                \
    X(RobotConfig, 0, getName)                                               \
    X(RobotConfig, 0, getCycletime)                                          \
    X(RobotConfig, 0, getRobotType)                                          \
    X(RobotConfig, 0, getRobotSubType)                                       \
    X(RobotConfig, 0, getControlBoxType)                                     \
    X(RobotConfig, 0, getDefaultToolAcc)                                     \
    X(RobotConfig, 0, getDefaultToolSpeed)                                   \
    X(RobotConfig, 0, getDefaultJointAcc)                                    \
    X(RobotConfig, 0, getDefaultJointSpeed)                                  \
    X(RobotConfig, 1, setMountingPose, pose)                                 \
    X(RobotConfig, 0, getMountingPose)                                       \
    X(RobotConfig, 1, setCollisionLevel, level);                             \
    X(RobotConfig, 0, getCollisionLevel);                                    \
    X(RobotConfig, 1, setCollisionStopType, type);                           \
    X(RobotConfig, 0, getCollisionStopType);                                 \
    X(RobotConfig, 1, setToolVoltageOutputDomain, domain);                   \
    X(RobotConfig, 0, getToolVoltageOutputDomain);                           \
    X(RobotConfig, 0, getTcpForceSensorNames)                                \
    X(RobotConfig, 1, selectTcpForceSensor, name)                            \
    X(RobotConfig, 0, hasTcpForceSensor)                                     \
    X(RobotConfig, 1, setTcpForceOffset, force_offset)                       \
    X(RobotConfig, 0, getTcpForceOffset)                                     \
    X(RobotConfig, 0, getBaseForceSensorNames)                               \
    X(RobotConfig, 1, selectBaseForceSensor, name)                           \
    X(RobotConfig, 0, hasBaseForceSensor)                                    \
    X(RobotConfig, 1, setBaseForceOffset, force_offset)                      \
    X(RobotConfig, 0, getBaseForceOffset)                                    \
    X(RobotConfig, 1, setPersistentParameters, param)                        \
    X(RobotConfig, 1, getKinematicsParam, real)                              \
    X(RobotConfig, 1, getKinematicsCompensate, ref_temperature)              \
    X(RobotConfig, 0, getSafetyParametersCheckSum)                           \
    X(RobotConfig, 1, confirmSafetyParameters, parameters)                   \
    X(RobotConfig, 0, getJointMaxPositions)                                  \
    X(RobotConfig, 0, getJointMinPositions)                                  \
    X(RobotConfig, 0, getJointMaxSpeeds)                                     \
    X(RobotConfig, 0, getJointMaxAccelerations)                              \
    X(RobotConfig, 0, getTcpMaxSpeeds)                                       \
    X(RobotConfig, 0, getTcpMaxAccelerations)                                \
    X(RobotConfig, 1, toolSpaceInRange, pose)                                \
    X(RobotConfig, 4, setPayload, m, cog, aom, inertia)                      \
    X(RobotConfig, 0, getPayload)                                            \
    X(RobotConfig, 0, getTcpOffset)                                          \
    X(RobotConfig, 0, getGravity)                                            \
    X(RobotConfig, 1, setGravity, gravity)                                   \
    X(RobotConfig, 1, setTcpOffset, offset)                                  \
    X(RobotConfig, 3, setToolInertial, m, com, inertial)                     \
    X(RobotConfig, 1, firmwareUpdate, fw)                                    \
    X(RobotConfig, 0, getFirmwareUpdateProcess)
// clang-format on
} // namespace common_interface
} // namespace arcs
#endif // AUBO_SDK_ROBOT_CONFIG_H
