#ifndef AUBO_SDK_SERIAL_INTERFACE_H
#define AUBO_SDK_SERIAL_INTERFACE_H

#include <vector>
#include <memory>

#include <aubo/type_def.h>
#include <aubo/global_config.h>

namespace arcs {
namespace common_interface {

class ARCS_ABI_EXPORT Serial
{
public:
    Serial();
    virtual ~Serial();

    /**
     * Open TCP/IP ethernet communication serial
     */
    int serialOpen(const std::string &device, int baud, float stop_bits,
                   int even, const std::string &serial_name = "serial_0");

    /**
     * Closes TCP/IP serial communication
     * Closes down the serial connection to the server.
     */
    int serialClose(const std::string &serial_name = "serial_0");

    /**
     * Reads a number of bytes from the serial. Bytes are in network byte
     * order. A maximum of 30 values can be read in one command.
     * A list of numbers read (list of ints, length=number+1)
     */
    int serialReadByte(const std::string &var,
                       const std::string &serial_name = "serial_0");

    /**
     * Reads a number of bytes from the serial. Bytes are in network byte
     * order. A maximum of 30 values can be read in one command.
     * A list of numbers read (list of ints, length=number+1)
     */
    int serialReadByteList(int number, const std::string &var,
                           const std::string &serial_name = "serial_0");

    /**
     * Reads all data from the serial and returns the data as a string.
     * Bytes are in network byte order.
     *
     * The optional parameters "prefix" and "suffix", can be used to express
     * what is extracted from the serial. The "prefix" specifies the start
     * of the substring (message) extracted from the serial. The data up to
     * the end of the "prefix" will be ignored and removed from the serial.
     * The "suffix" specifies the end of the substring (message) extracted
     * from the serial. Any remaining data on the serial, after the "suffix",
     * will be preserved. E.g. if the serial server sends a string
     * "noise>hello<", the controller can receive the "hello" by calling this
     * script function with the prefix=">" and suffix="<". By using the
     * "prefix" and "suffix" it is also possible send multiple string to the
     * controller at once, because the suffix defines where the message ends.
     * E.g. sending ">hello<>world<"
     */
    int serialReadString(const std::string &var,
                         const std::string &serial_name = "serial_0",
                         const std::string &prefix = "",
                         const std::string &suffix = "",
                         bool interpret_escape = false);

    /**
     * Sends a byte to the server
     * Sends the byte <value> through the serial. Expects no response. Can
     * be used to send special ASCII characters; 10 is newline, 2 is start of
     * text, 3 is end of text.
     */
    int serialSendByte(char value, const std::string &serial_name = "serial_0");

    /**
     * Sends an int (int32_t) to the server
     * Sends the int <value> through the serial. Send in network byte order.
     * Expects no response
     */
    int serialSendInt(int value, const std::string &serial_name = "serial_0");

    /**
     * Sends a string with a newline character to the server - useful for
     * communicatin with the UR dashboard server
     * Sends the string <str> through the serial in ASCII coding. Expects no
     * response.
     */
    int serialSendLine(const std::string &str,
                       const std::string &serial_name = "serial_0");

    /**
     * Sends a string to the server
     * Sends the string <str> through the serial in ASCII coding. Expects no
     * response.
     */
    int serialSendString(const std::string &str,
                         const std::string &serial_name = "serial_0");
    int serialSendAllString(bool is_check, const std::vector<char> &str,
                            const std::string &serial_name = "serial_0");

protected:
    void *d_;
};
using SerialPtr = std::shared_ptr<Serial>;

// clang-format off
#define SERIAL_DECLARES                                                                \
    X(Serial, 5, serialOpen, device, baud, stop_bits, even, serial_name)               \
    X(Serial, 1, serialClose, serial_name)                                             \
    X(Serial, 2, serialReadByte, var, serial_name)                                     \
    X(Serial, 3, serialReadByteList, number, var, serial_name)                         \
    X(Serial, 5, serialReadString, var, serial_name, prefix, suffix, interpret_escape) \
    X(Serial, 2, serialSendByte, value, serial_name)                                   \
    X(Serial, 2, serialSendInt, value, serial_name)                                    \
    X(Serial, 2, serialSendLine, str, serial_name)                                     \
    X(Serial, 2, serialSendString, str, serial_name)                                   \
    X(Serial, 3, serialSendAllString, is_check, str, serial_name)
// clang-format on

} // namespace common_interface
} // namespace arcs
#endif
